@@ -1,3 +1,4 @@
+import os, sys
 import calendar
 import numpy as np
 import pandas as pd
@@ -40,7 +41,7 @@ class TrackPlot(Plot):
         
         self.use_credit = True
     
-    def plot_storm(self,storm,domain="dynamic",plot_all=False,ax=None,return_ax=False,track_labels=False,prop={},map_prop={}):
+    def plot_storm(self,storm,domain="dynamic",plot_all=False,ax=None,return_ax=False,track_labels=False,save_path=None,prop={},map_prop={}):
         
         r"""
         Creates a plot of a single storm track.
@@ -68,7 +69,7 @@ class TrackPlot(Plot):
         """
         
         #Set default properties
-        default_prop={'dots':True,'fillcolor':'category','linecolor':'k','category_colors':'default','linewidth':1.0,'ms':7.5}
+        default_prop={'dots':True,'fillcolor':'category','linecolor':'k','category_colors':'default','linewidth':1.0,'ms':7.5,'title':True}
         default_map_prop={'res':'m','land_color':'#FBF5EA','ocean_color':'#EDFBFF','linewidth':0.5,'linecolor':'k','figsize':(14,9),'dpi':200}
         
         #Initialize plot
@@ -210,17 +211,25 @@ class TrackPlot(Plot):
         type_array = np.array(storm_data['type'])
         idx = np.where((type_array == 'SD') | (type_array == 'SS') | (type_array == 'TD') | (type_array == 'TS') | (type_array == 'HU'))
         tropical_vmax = np.array(storm_data['vmax'])[idx]
+        
+        #Coerce to include non-TC points if storm hasn't been designated yet
+        add_ptc_flag = False
+        if len(tropical_vmax) == 0:
+            add_ptc_flag = True
+            idx = np.where((type_array == 'LO') | (type_array == 'DB'))
+        tropical_vmax = np.array(storm_data['vmax'])[idx]
             
         subtrop = classify_subtropical(np.array(storm_data['type']))
         peak_idx = storm_data['vmax'].index(np.nanmax(tropical_vmax))
         peak_basin = storm_data['wmo_basin'][peak_idx]
         storm_type = get_storm_classification(np.nanmax(tropical_vmax),subtrop,peak_basin)
+        if add_ptc_flag == True: storm_type = "Potential Tropical Cyclone"
         self.ax.set_title(f"{storm_type} {storm_data['name']}",loc='left',fontsize=17,fontweight='bold')
 
         #Add right title
         ace = storm_data['ace']
+        if add_ptc_flag == True: ace = 0.0
         type_array = np.array(storm_data['type'])
-        idx = np.where((type_array == 'SD') | (type_array == 'SS') | (type_array == 'TD') | (type_array == 'TS') | (type_array == 'HU'))
         
         #Get storm extrema for display
         mslp_key = 'mslp' if 'wmo_mslp' not in storm_data.keys() else 'wmo_mslp'
@@ -264,7 +273,11 @@ class TrackPlot(Plot):
             c4 = mlines.Line2D([], [], linestyle='None', ms=prop['ms'], mec='k',mew=0.5, label='Category 4', marker='o', color=get_colors_sshws(113))
             c5 = mlines.Line2D([], [], linestyle='None', ms=prop['ms'], mec='k',mew=0.5, label='Category 5', marker='o', color=get_colors_sshws(137))
             self.ax.legend(handles=[ex,sb,td,ts,c1,c2,c3,c4,c5], prop={'size':11.5})
-
+        
+        #Save image if specified
+        if save_path != None and isinstance(save_path,str) == True:
+            plt.savefig(os.path.join(save_path,f"{storm_data['name']}_{storm_data['year']}_track.png"),bbox_inches='tight')
+        
         #Return axis if specified, otherwise display figure
         if ax != None or return_ax == True:
             return self.ax
@@ -272,7 +285,7 @@ class TrackPlot(Plot):
             plt.show()
             plt.close()
     
-    def plot_storms(self,storms,domain="dynamic",title_text="TC Track Composite",filter_dates=('1/1','12/31'),plot_all_dots=False,ax=None,return_ax=False,prop={},map_prop={}):
+    def plot_storms(self,storms,domain="dynamic",title_text="TC Track Composite",filter_dates=('1/1','12/31'),plot_all_dots=False,ax=None,return_ax=False,save_path=None,prop={},map_prop={}):
         
         r"""
         Creates a plot of multiple storm tracks.
@@ -453,13 +466,19 @@ class TrackPlot(Plot):
             c5 = mlines.Line2D([], [], linestyle='None', ms=prop['ms'], mec='k',mew=0.5, label='Category 5', marker='o', color=get_colors_sshws(137))
             self.ax.legend(handles=[ex,sb,td,ts,c1,c2,c3,c4,c5], prop={'size':11.5})
 
+        #Save image if specified
+        if save_path != None and isinstance(save_path,str) == True:
+            plt.savefig(os.path.join(save_path,f"tropycal_track_composite.png"),bbox_inches='tight')
+        
         #Return axis if specified, otherwise display figure
+        if return_ax == True:
             return self.ax,'/'.join([str(i) for i in [bound_w,bound_e,bound_s,bound_n]])
         else:
             plt.show()
             plt.close()
         
-    def plot_storm_nhc(self,forecast,track=None,track_labels='fhr',cone_days=5,domain="dynamic_forecast",ax=None,return_ax=False,prop={},map_prop={}):
+    def plot_storm_nhc(self,forecast,track=None,track_labels='fhr',cone_days=5,domain="dynamic_forecast",ax=None,return_ax=False,save_path=
+None,prop={},map_prop={}):
         
         r"""
         Creates a plot of the operational NHC forecast track along with observed track data.
@@ -493,9 +512,6 @@ class TrackPlot(Plot):
             Property of cartopy map.
         """
         
-        #Determine if forecast is realtime
-        realtime_flag = True if forecast['advisory_num'] == -1 else False
-        
         #Set default properties
         default_prop={'dots':True,'fillcolor':'category','linecolor':'k','category_colors':'default','linewidth':1.0,'ms':7.5,'cone_lw':1.0,'cone_alpha':0.6}
         default_map_prop={'res':'m','land_color':'#FBF5EA','ocean_color':'#EDFBFF','linewidth':0.5,'linecolor':'k','figsize':(14,9),'dpi':200}
@@ -553,7 +569,6 @@ class TrackPlot(Plot):
                 fcst_hr = np.array(forecast['fhr'])
                 start_slice = 0
                 if 3 in fcst_hr: start_slice = 3
-                if realtime_flag == True: start_slice = int(fcst_hr[1])
                 iter_hr = np.array(forecast['fhr'])[fcst_hr>=start_slice][0]
                 fcst_lon = np.array(forecast['lon'])[fcst_hr>=start_slice][0]
                 fcst_lat = np.array(forecast['lat'])[fcst_hr>=start_slice][0]
@@ -642,7 +657,6 @@ class TrackPlot(Plot):
         fcst_hr = np.array(forecast['fhr'])
         start_slice = 0
         if 3 in fcst_hr: start_slice = 3
-        if realtime_flag == True: start_slice = int(fcst_hr[1])
         check_duration = fcst_hr[(fcst_hr>=start_slice) & (fcst_hr<=cone_days*24)]
 
         #Check for sufficiently many hours
@@ -830,24 +844,18 @@ class TrackPlot(Plot):
         fcst_hr = forecast['fhr']
         start_slice = 0
         if 3 in fcst_hr: start_slice = 1
-        if realtime_flag == True: start_slice = 1
         forecast_date = (forecast['init']+timedelta(hours=fcst_hr[start_slice])).strftime("%H%M UTC %d %b %Y")
         forecast_id = forecast['advisory_num']
         
-        #Get wind value to display
-        if first_fcst_wind == "N/A":
-            wind_display_value = "N/A"
-        else:
-            wind_display_value = knots_to_mph(first_fcst_wind)
-        
         if forecast_id == -1:
-            title_text = f"Current Intensity: {wind_display_value} mph {dot} {first_fcst_mslp} hPa"
+            title_text = f"Current Intensity: {knots_to_mph(first_fcst_wind)} mph {dot} {first_fcst_mslp} hPa"
             if 'cone' in forecast.keys() and forecast['cone'] == False:
                 title_text += f"\nJTWC Issued: {forecast_date}"
             else:
                 title_text += f"\nNHC Issued: {forecast_date}"
         else:
-            title_text = f"{wind_display_value} mph {dot} {first_fcst_mslp} hPa {dot} Forecast #{forecast_id}"
+            if first_fcst_wind != "N/A": first_fcst_wind = knots_to_mph(first_fcst_wind)
+            title_text = f"{first_fcst_wind} mph {dot} {first_fcst_mslp} hPa {dot} Forecast #{forecast_id}"
             title_text += f"\nForecast Issued: {forecast_date}"
         
         
@@ -880,10 +888,7 @@ class TrackPlot(Plot):
         except:
             warning_text = ""
         try:
-            if cone['year'] >= 2008:
-                warning_text += f"The cone of uncertainty in this product was generated internally using {cone['year']} official\nNHC cone radii. This cone differs slightly from the official NHC cone.\n\n"
-            else:
-                warning_text += f"The cone of uncertainty in this product was generated internally using {cone['year']} derived\nNHC cone radii. This cone differs from official NHC products at the time.\n\n"
+            warning_text += f"The cone of uncertainty in this product was generated internally using {cone['year']} official\nNHC cone radii. This cone differs slightly from the official NHC cone.\n\n"
         except:
             pass
         
@@ -893,6 +898,10 @@ class TrackPlot(Plot):
         credit_text = self.plot_credit()
         self.add_credit(credit_text)
         
+        #Save image if specified
+        if save_path != None and isinstance(save_path,str) == True:
+            plt.savefig(os.path.join(save_path,f"{storm_data['name']}_{storm_data['year']}_track.png"),bbox_inches='tight')
+        
         #Return axis if specified, otherwise display figure
         if ax != None or return_ax == True:
             return self.ax
@@ -1107,9 +1116,6 @@ class TrackPlot(Plot):
         
         """
 
-        #Determine if forecast is realtime
-        realtime_flag = True if forecast['advisory_num'] == -1 else False
-        
         #Source: https://www.nhc.noaa.gov/verification/verify3.shtml
         #Radii are in nautical miles
         cone_climo_hr = [3,12,24,36,48,72,96,120]
@@ -1127,24 +1133,6 @@ class TrackPlot(Plot):
         cone_size_atl[2010] = [16,36,62,85,108,161,220,285]
         cone_size_atl[2009] = [16,36,62,89,111,167,230,302]
         cone_size_atl[2008] = [16,39,67,92,118,170,233,305]
-        cone_size_atl[2007] = [16,39,69,98,124,178,253,324]
-        cone_size_atl[2006] = [16,42,73,103,131,192,259,335]
-        cone_size_atl[2005] = [16,43,77,109,142,207,266,350]
-        cone_size_atl[2004] = [16,46,81,117,156,218,275,369]
-        cone_size_atl[2003] = [16,47,85,122,162,227,318,433]
-        cone_size_atl[2002] = [16,48,85,123,164,233,316,443]
-        cone_size_atl[2001] = [16,48,85,124,162,227]
-        cone_size_atl[2000] = [16,50,91,132,170,244]
-        cone_size_atl[1999] = [16,52,96,136,178,260]
-        cone_size_atl[1998] = [16,53,98,141,184,273]
-        cone_size_atl[1997] = [16,54,99,144,191,281]
-        cone_size_atl[1996] = [16,55,108,155,206,312]
-        cone_size_atl[1995] = [16,60,113,166,217,358]
-        cone_size_atl[1994] = [16,59,113,168,220,344]
-        cone_size_atl[1993] = [16,58,108,162,217,343]
-        cone_size_atl[1992] = [16,59,112,166,223,343]
-        cone_size_atl[1991] = [16,59,112,167,225,353]
-        cone_size_atl[1990] = [16,57,111,168,232,356]
 
         cone_size_pac = {}
         cone_size_pac[2020] = [16,25,38,51,65,78,91,115,138]
@@ -1164,8 +1152,6 @@ class TrackPlot(Plot):
         #Fix for 2020 that now incorporates 60 hour forecasts
         if forecast['init'].year >= 2020:
             cone_climo_hr = [3,12,24,36,48,60,72,96,120]
-        if realtime_flag == True: #Realtime
-            cone_climo_hr = [forecast['fhr'][1],12,24,36,48,60,72,96,120]
 
         #Function for interpolating between 2 times
         def temporal_interpolation(value, orig_times, target_times):
@@ -1256,24 +1242,18 @@ class TrackPlot(Plot):
                 cone_size = 0
                 #raise RuntimeError("Error: No cone information is available for the requested basin.")
         else:
+            cone_year = 2008
+            warnings.warn(f"No cone information is available for the requested year. Defaulting to 2008 cone.")
             if forecast['basin'] == 'north_atlantic':
-                cone_year = 1990
-                cone_size = cone_size_atl[1990]
-                msg = f"No cone information is available for the requested year. Defaulting to 1990 cone."
-                warnings.warn(msg)
+                cone_size = cone_size_atl[2008]
             elif forecast['basin'] == 'east_pacific':
-                cone_year = 2008
                 cone_size = cone_size_pac[2008]
-                msg = f"No cone information is available for the requested year. Defaulting to 2008 cone."
-                warnings.warn(msg)
             else:
-                cone_year = 2008
                 cone_size = 0
                 #raise RuntimeError("Error: No cone information is available for the requested basin.")
             #raise RuntimeError("Error: No cone information is available for the requested year.")
         
-        #Check if fhr3 is available (or 1st hour for realtime), then get forecast data
-        check_fhr = forecast['fhr'][1] if realtime_flag == True else 3
+        #Check if fhr3 is available, then get forecast data
         flag_12 = 0
         if forecast['fhr'][0] == 12:
             flag_12 = 1
@@ -1285,13 +1265,13 @@ class TrackPlot(Plot):
             subtract_by = t[0]
             t = t - t[0]
             interp_fhr_idx = np.arange(t[0],t[-1]+0.1,0.1) - t[0]
-        elif check_fhr in forecast['fhr'] and 1 in forecast['fhr'] and 0 in forecast['fhr']:
+        elif 3 in forecast['fhr'] and 1 in forecast['fhr'] and 0 in forecast['fhr']:
             fcst_lon = forecast['lon'][2:]
             fcst_lat = forecast['lat'][2:]
             fhr = forecast['fhr'][2:]
             t = np.array(fhr)/6.0
             interp_fhr_idx = np.arange(t[0],t[-1]+0.01,0.1)
-        elif check_fhr in forecast['fhr'] and 0 in forecast['fhr']:
+        elif 3 in forecast['fhr'] and 0 in forecast['fhr']:
             idx = np.array([i for i,j in enumerate(forecast['fhr']) if j in cone_climo_hr])
             fcst_lon = np.array(forecast['lon'])[idx]
             fcst_lat = np.array(forecast['lat'])[idx]
@@ -1357,12 +1337,6 @@ class TrackPlot(Plot):
         interp_rad = np.apply_along_axis(lambda n: temporal_interpolation(n,fhr,interp_fhr),axis=0,arr=temp_arr)
 
         #Initialize 0.05 degree grid
-        grid_extent = 9
-        if cone_year >= 2004: grid_extent = 8.5
-        if cone_year >= 2008: grid_extent = 8
-        if cone_year >= 2012: grid_extent = 7.5
-        if cone_year >= 2016: grid_extent = 7
-        
         gridlats = np.arange(min(interp_lat)-7,max(interp_lat)+7,0.05)
         gridlons = np.arange(min(interp_lon)-7,max(interp_lon)+7,0.05)
         gridlons2d,gridlats2d = np.meshgrid(gridlons,gridlats)
@@ -1452,19 +1426,22 @@ class TrackPlot(Plot):
         for key, val in pos.items():
             pos[key] = (val*scale) + shift
 
+        #Apply coordinate transform
+        transform = ccrs.PlateCarree()._as_mpl_transform(self.ax)
+        
         start = False
         for label, data_str in G.edges():
             if start == False:
                 start = True
                 continue
-            self.ax.annotate(label,
-                        xy=pos[data_str], xycoords='data',
-                        xytext=pos[label], textcoords='data', fontweight='bold', ha='center', va='center',
+            self.ax.annotate(label, #xycoords="data"
+                        xy=pos[data_str], xycoords=transform,
+                        xytext=pos[label], textcoords=transform, fontweight='bold', ha='center', va='center',
                         arrowprops=dict(arrowstyle="-",#->
                                         shrinkA=0, shrinkB=0,
                                         connectionstyle="arc3", 
                                         color='k'),
-                        transform=ccrs.PlateCarree())
+                        transform=ccrs.PlateCarree(),clip_on=True)
 
     def plot_gridded(self,xcoord,ycoord,zcoord,VEC_FLAG=False,domain="north_atlantic",ax=None,return_ax=False,prop={},map_prop={}):
         
@@ -1494,7 +1471,8 @@ class TrackPlot(Plot):
         
         #Set default properties
         default_prop={'cmap':'category','levels':None,\
-                      'left_title':'','right_title':'All storms'}
+                      'left_title':'','right_title':'All storms',
+                      'plot_values':False,'values_size':None}
         default_map_prop={'res':'m','land_color':'#FBF5EA','ocean_color':'#EDFBFF','linewidth':0.5,'linecolor':'k','figsize':(14,9),'dpi':200}
         
         #Initialize plot
@@ -1567,7 +1545,8 @@ class TrackPlot(Plot):
             y_center = (ycoord[:-1,:-1]+ycoord[1:,1:])*.5
             u = zcoord[0][:-1,:-1]
             v = zcoord[1][:-1,:-1]
-            self.ax.quiver(x_center,y_center,u,v,color='w',alpha=0.6,transform=ccrs.PlateCarree(),\
+            if not prop['plot_values']:
+                self.ax.quiver(x_center,y_center,u,v,color='w',alpha=0.6,transform=ccrs.PlateCarree(),\
                            pivot='mid',width=.001*binsize,headwidth=3.5,headlength=4.5,headaxislength=4)
             zcoord = vecmag
         
@@ -1585,6 +1564,23 @@ class TrackPlot(Plot):
             else:
                 cbmap = self.ax.pcolor(xcoord,ycoord,zcoord,cmap=cmap,vmin=vmin,vmax=vmax,
                                        transform=ccrs.PlateCarree())
+        if prop['plot_values']:
+            binsize = abs(xcoord[0,0]-xcoord[0,1])
+            x_center = (xcoord[:-1,:-1]+xcoord[1:,1:])*.5
+            y_center = (ycoord[:-1,:-1]+ycoord[1:,1:])*.5
+            xs = x_center.flatten(order='C')
+            ys = y_center.flatten(order='C')
+            zs = zcoord[:-1,:-1].flatten(order='C')
+            if prop['values_size'] is None:
+                fs = binsize*4
+            else:
+                fs = prop['values_size']
+            for xtext,ytext,ztext in zip(xs,ys,zs):
+                if not np.isnan(ztext) and xtext%360>bound_w%360 and xtext%360<bound_e%360 and\
+                    ytext>bound_s and ytext<bound_n:
+                    self.ax.text(xtext,ytext,ztext.astype(int),ha='center',va='center',fontsize=fs,\
+                                     color='w',alpha=0.8,transform=ccrs.PlateCarree())
+                
 
         #--------------------------------------------------------------------------------------
 
